/**
 *
 * Compiz galaxy livewallpaper plugin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2011 Maximilian Schnarr <Fyrmir@gmx.de>
 *
 *
 * Several parts taken from the original plugin developed by Pal Dorogi
 * with the following copyright:
 *
 * Copyright (C) 2010 Pal Dorogi <pal.dorogi@gmail.com>
 *
 * Original plugin can be found here:
 * http://launchpad.net/compiz-plugins-livewallpapers
 *
 **/

#ifndef _PARTICLE_H_
#define _PARTICLE_H_

#include <core/core.h>
#include <composite/composite.h>
#include <opengl/opengl.h>

/*
 * Represents a star.
 */
class Particle
{
	public:
		/* Initialises the particle with random values */
		Particle();

		/*
		 * Those values needed for the general parametic form of an
		 * ellipse. Phi is the angle between the x-axis and the major
		 * axis of the ellipse.
		 */
		GLfloat cos_phi, sin_phi;

		/*
		 * The current angle of the particle. It represents the position
		 * of this particle.
		 */
		GLfloat angle;

		/* The z value of the real position of this star. */
		GLfloat z;

		/* The distance from the center of the galaxy to this particle. */
		GLfloat distance;

		/* The angular velocity of the particle in 1/ms. */
		GLfloat speed;

		/* Color of the particle. */
		GLubyte r, g, b, a;
};

/*
 * Manages a great amount of stars.
 */
class SpiralGalaxy
{
	private:
		/* The amount of stars. */
		unsigned int starsCount;

		/* The size of one star. */
		GLint starsSize;

		/* The texture of each star. */
		GLTexture::List flareTexture;
		CompSize flareSize;

		/* An array holding all stars. */
		std::vector<Particle> stars;

		/*
		 * Arrays used to draw many particles. The updateParticles method
		 * fills those arrays with values.
		 */
		std::vector<GLfloat> vertices;
		std::vector<GLubyte> colors;

	public:
		SpiralGalaxy();

		/*
		 * Allocates all the memory required to draw starsCount
		 * stars. It can throw an std::bad_alloc exception!
		 */
		void setStarsCount(unsigned int);

		/* Get the current amount of stars. */
		unsigned int getStarsCount();

		/* Set the size of the stars. */
		void setStarsSize(int);

		/* Get the stars size. */
		int getStarsSize();

		/*
		 * Generate the flare texture. If the params is true, it will
		 * draw streaks.
		 */
		void generateFlare(bool);

		/*
		 * Update the particles, move them and do some work before
		 * drawing. Pass time in ms.
		 */
		void updateParticles(int);

		/*
		 * Draw the particles using opengl. Set first argument to true
		 * if you want to render particle by particle.
		 */
		void drawParticles(bool);
};

#endif /* _PARTICLE_H_ */
